/*
 * Copyright (c) 2014, Oracle and/or its affiliates. All Rights Reserved.
 */

// Santiago Pericas-Geertsen <santiago.pericasgeertsen@oracle.com>
// Irfan Ahmed <irfan.ahmed@oracle.com>

module.exports = function(backend) {
    var avatar = require("org/glassfish/avatar");

    avatar.registerRestService({
        url: "/rest/families/list"
    }, function() {
        this.onGet = function(req, res) {
            backend.Family.findAll().then(function(fs) {
                if (!fs) {
                    res.status = 404;
                    res.send('Unable to list families');
                } else {
                    res.send(fs);
                }
            }).otherwise(function() {
                res.status = 500;
                res.send("Error listing familiesfs");
            });
        };
    });
    
    avatar.registerRestService({
        url: "/rest/families/add"
    }, function() {
        this.onPut = function(req, res) {
            backend.Family.create(req.data).then(function() {
                // family exists
                res.send("done");
            }).otherwise(function(f) {
                res.status = 500;
                res.send({error: JSON.stringify(f)});
            });
        };
    });
    
    avatar.registerRestService({
        url: "/rest/family/{fname}"
    }, function() {
        this.onGet = function(req, res) {
            var fname = this.fname;
            backend.Family.find(fname).then(function(f) {
                if (!f) {
                    res.status = 404;
                    res.send('Unable to find family ' + fname);
                } else {
                    res.send(f);
                }
            }).otherwise(function() {
                res.status = 500;
                res.send("Error getting family " + fname);
            });
        };        
    });

    avatar.registerRestService({
        url: "/rest/family/{fname}/products"
    }, function() {
        this.onGet = function(req, res) {
            var fname = this.fname;
            backend.Family.findProducts(fname).then(function(ps) {
                if (!ps) {
                    res.status = 404;
                    res.send('Unable to find family ' + fname);
                } else {
                    res.send(ps);
                }
            }).otherwise(function() {
                res.status = 500;
                res.send("Error getting family " + fname);
            });
        };
    });
};