<?xml version="1.0" encoding="US-ASCII"?>
<!--

    DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.

    Copyright (c) 2013 Oracle and/or its affiliates. All rights reserved.

    The contents of this file are subject to the terms of the GNU General
    Public License Version 2 only ("GPL"). You may not use this file except
    in compliance with the License.  You can obtain a copy of the License at
    https://avatar.java.net/license.html or legal/LICENSE.txt.  See the
    License for the specific language governing permissions and limitations
    under the License.

    When distributing the software, include this License Header Notice in each
    file and include the License file at legal/LICENSE.txt.

    GPL Classpath Exception:
    Oracle designates this particular file as subject to the "Classpath"
    exception as provided by Oracle in the GPL Version 2 section of the License
    file that accompanied this code.

    Modifications:
    If applicable, add the following below the License Header, with the fields
    enclosed by brackets [] replaced by your own identifying information:
    "Portions Copyright [year] [name of copyright owner]"

-->

<!--
  - Generator for Reference Documentation
  - @author spericas@java.net
-->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0"
                xmlns:a="http://www.glassfish.org/avatar"
                xmlns:util="http://xml.apache.org/xalan/java/org.glassfish.avatar.util.GenReferenceDocUtil"
                exclude-result-prefixes="xsl util a">

    <xsl:param name="widgetLib" select="'jquery'"/>
    <xsl:param name="fileName" select="''"/>
    
    <xsl:output method="xml" indent="yes" omit-xml-declaration="yes"/>

    <xsl:strip-space elements="a:definition"/>

    <xsl:template match="/">
        <xsl:if test="not(a:viewModule[@ifOption]) and $widgetLib = 'jquery'">
            <xsl:call-template name="generateDoc"/>
        </xsl:if>
        <xsl:if test='util:contains(a:viewModule/@ifOption, concat("&apos;", concat($widgetLib, "&apos;")))'>
            <xsl:call-template name="generateDoc"/>
        </xsl:if>
    </xsl:template>

    <xsl:template name="generateDoc">
        <dl>
            <xsl:apply-templates/>
        </dl>
    </xsl:template>


    <xsl:template match="a:definition">
        <xsl:call-template name="generateDocDef"/>
    </xsl:template>

    <!--
      - Generate documentation for a node.
      -->
    <xsl:template name="generateDocDef">
        <!-- Ignore internal HTML widgets -->
        <xsl:if test="not(util:startsWith(@id, 'html') or util:startsWith(@id, '$'))">
            <dt>
                <h4 class="name" id="{@id}">
                    <span class="type-signature">Definition </span>
                    <xsl:value-of select="@id"/>
                    <span class="signature"> (<xsl:value-of select="$fileName"/>)</span>
                </h4>
            </dt>
            <dd>
                <div class="description">
                    <xsl:call-template name="formatDoc"/>
                </div>
                <xsl:choose>
                    <xsl:when test="a:property">
                        <h5>Properties:</h5>
                        <br/>
                        <table class="params">
                            <thead>
                                <tr>
                                    <th>Name</th>
                                    <th>Type</th>
                                    <th>i18n</th>
                                    <th>Default</th>
                                    <th class="last">Description</th>
                                </tr>
                            </thead>
                            <tbody>
                                <xsl:for-each select="a:property">
                                    <tr>
                                        <td class="name">
                                            <code>
                                                <xsl:value-of select="@name"/>
                                            </code>
                                        </td>
                                        <td class="type">
                                            <span class="param-type">
                                                <xsl:value-of select="@type"/>
                                            </span>
                                        </td>
                                        <td class="i18n">
                                            <span class="param-type">
                                                <xsl:choose>
                                                    <xsl:when test="@i18n">
                                                        <xsl:value-of select="@i18n"/>
                                                    </xsl:when>
                                                    <xsl:otherwise>
                                                        false
                                                    </xsl:otherwise>
                                                </xsl:choose>
                                            </span>
                                        </td>
                                        <td class="default">
                                            <span class="param-type">
                                                <xsl:choose>
                                                    <xsl:when test="@default">
                                                        <xsl:value-of select="@default"/>
                                                    </xsl:when>
                                                    <xsl:otherwise>n/a</xsl:otherwise>
                                                </xsl:choose>
                                            </span>
                                        </td>
                                        <td class="description last">
                                            <xsl:call-template name="formatDoc"/>
                                        </td>
                                    </tr>
                                </xsl:for-each>
                            </tbody>
                        </table>
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:if test="not(a:function)">
                            <h5>No Properties</h5>
                        </xsl:if>
                    </xsl:otherwise>
                </xsl:choose>
                <xsl:if test="a:example">
                    <p>
                    <xsl:for-each select="a:example">
                        <div class="example">
                            <h5>Example: <xsl:value-of select="@type"/></h5>
                            <xsl:call-template name="formatExample"/>
                        </div>
                    </xsl:for-each>
                    </p>
                </xsl:if>
            </dd><br/>
        </xsl:if>
    </xsl:template>

    <!--
      - Format documentation text for a node or property.
      -->
    <xsl:template name="formatDoc">
        <xsl:choose>
            <xsl:when test="count(text()) > 0">
                <xsl:for-each select="node()">
                    <xsl:if test="namespace-uri() = ''">   <!-- HTML markup -->
                        <xsl:copy-of select="."/>
                    </xsl:if>
                </xsl:for-each>
            </xsl:when>
            <xsl:otherwise>TODO DOC</xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <!--
      - Format documentation for an example. An example's content model is of
      - the form (pre | p)+.
      -->
    <xsl:template name="formatExample">
        <xsl:for-each select="pre | p">
            <xsl:choose>
                <xsl:when test="name() = 'pre'">
                    <pre class="prettyprint">
                        <code><xsl:value-of select="text()"/></code>
                    </pre>
                </xsl:when>
                <xsl:otherwise>
                    <p class="code-caption">
                        <xsl:copy-of select="node()"/>
                    </p>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:for-each>
    </xsl:template>

    <!--
      - Ignore comments
      -->
    <xsl:template match="comment()">
        <!-- no-op -->
    </xsl:template>

    <!--
      - Ignore arbitrary text nodes.
      -->
    <xsl:template match="text()">
        <!-- no-op -->
    </xsl:template>

</xsl:stylesheet>
