<?xml version="1.0" encoding="US-ASCII"?>
<!--

    DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.

    Copyright (c) 2013 Oracle and/or its affiliates. All rights reserved.

    The contents of this file are subject to the terms of the GNU General
    Public License Version 2 only ("GPL"). You may not use this file except
    in compliance with the License.  You can obtain a copy of the License at
    https://avatar.java.net/license.html or legal/LICENSE.txt.  See the
    License for the specific language governing permissions and limitations
    under the License.

    When distributing the software, include this License Header Notice in each
    file and include the License file at legal/LICENSE.txt.

    GPL Classpath Exception:
    Oracle designates this particular file as subject to the "Classpath"
    exception as provided by Oracle in the GPL Version 2 section of the License
    file that accompanied this code.

    Modifications:
    If applicable, add the following below the License Header, with the fields
    enclosed by brackets [] replaced by your own identifying information:
    "Portions Copyright [year] [name of copyright owner]"

-->

<!--
  - Avatar XML to HTML transformation
  - @author spericas@java.net
  -
  - The purpose of this stylesheet is to aid with the conversion of existing
  - XML samples into HTML. See the inverse transform html2xml for more info.
  - It is intended to be used "by hand" to avoid the repetitive conversions.
-->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0"
                xmlns:a="http://www.glassfish.org/avatar"
                xmlns:x="http://www.w3.org/1999/xhtml"
                xmlns:util="http://xml.apache.org/xalan/java/org.glassfish.avatar.util.Html2XmlUtil"
                exclude-result-prefixes="util a x">

    <!-- Output of transformation is XML -->
    <xsl:output method="html" indent="no"/>

    <!-- Name of HTML file excluding path but including extension -->
    <xsl:param name="fileName" select="'unnamed'"/>

    <!-- Default page name based on file name -->
    <xsl:variable name="pageName" select="substring-before($fileName, '.')"/>
    
    <!-- Avatar namespace -->
    <xsl:variable name="avatarNs" select="'http://www.glassfish.org/avatar'"/>

    <xsl:template match="/a:viewModule">
        <html>
            <xsl:if test="count(a:page) = 0">
                <xsl:attribute name="data-newpage">
                    <xsl:value-of select="'false'"/>
                </xsl:attribute>
            </xsl:if>
            <head>
                <title>
                    <xsl:value-of select="//a:page/@title"/>
                </title>
            </head>
            <body>
                <xsl:call-template name="processDataModels"/>
                <xsl:apply-templates/>
            </body>
        </html>
    </xsl:template>

    <xsl:template name="processDataModels">
        <xsl:for-each select="*">
            <xsl:if test="util:stringEndsWith(local-name(), 'Model')">
                <script data-model="{local-name()}">
                    <xsl:call-template name="processAllAttrs"/>
                    <xsl:if test="namespace-uri() != $avatarNs">
                        <xsl:attribute name="data-namespace">
                            <xsl:value-of select="namespace-uri()"/>
                        </xsl:attribute>
                    </xsl:if>
                    <xsl:value-of select="."/>
                </script>
            </xsl:if>
            <xsl:if test="local-name() = 'uiController'">
                <script data-controller="{local-name()}">
                    <xsl:call-template name="processAllAttrs"/>
                    <xsl:if test="namespace-uri() != $avatarNs">
                        <xsl:attribute name="data-namespace">
                            <xsl:value-of select="namespace-uri()"/>
                        </xsl:attribute>
                    </xsl:if>
                    <xsl:value-of select="."/>
                </script>
            </xsl:if>
        </xsl:for-each>
    </xsl:template>

    <xsl:template match="a:model">
        <script data-type="{@idref}">
            <xsl:if test="@id">
                <xsl:attribute name="data-instance">
                    <xsl:value-of select="@id"/>
                </xsl:attribute>
            </xsl:if>
            <xsl:call-template name="processAllAttrs">
                <xsl:with-param name="skipAttrs" select="'id idref'"/>
            </xsl:call-template>
        </script>
    </xsl:template>

    <xsl:template match="a:controller">
        <script data-controller-type="{@idref}">
            <xsl:if test="@id">
                <xsl:attribute name="data-instance">
                    <xsl:value-of select="@id"/>
                </xsl:attribute>
            </xsl:if>
            <xsl:call-template name="processAllAttrs">
                <xsl:with-param name="skipAttrs" select="'id idref'"/>
            </xsl:call-template>
        </script>
    </xsl:template>

    <xsl:template match="a:input | a:checkbox">
        <label for="{concat($pageName, generate-id(.))}">
            <xsl:value-of select="@label"/>
        </label>
        <xsl:text>&#10;</xsl:text>
        <input id="{concat($pageName, generate-id(.))}">
            <xsl:if test="local-name() = 'checkbox'">
                <xsl:attribute name="type">checkbox</xsl:attribute>
            </xsl:if>
            <xsl:call-template name="processAllAttrs">
                <xsl:with-param name="skipAttrs" select="'label'"/>
            </xsl:call-template>
        </input>
    </xsl:template>

    <xsl:template match="a:select">
        <xsl:variable name="id" select="concat($pageName, generate-id(.))"/>
        <label for="{$id}">
            <xsl:value-of select="@label"/>
        </label>
        <xsl:text>&#10;</xsl:text>
        <select id="{$id}">
            <xsl:call-template name="processAllAttrs">
                <xsl:with-param name="skipAttrs" select="'label'"/>
            </xsl:call-template>
            <xsl:text>&#10;</xsl:text>
            <xsl:for-each select="a:option">
                <option value="{@value}">
                    <xsl:value-of select="@label"/>
                </option>
                <xsl:text>&#10;</xsl:text>
            </xsl:for-each>
        </select>
    </xsl:template>

    <xsl:template match="a:output">
        <span>
            <xsl:call-template name="processAllAttrs">
                <xsl:with-param name="skipAttrs" select="'value'"/>
            </xsl:call-template>
            <xsl:value-of select="@value"/>
        </span>
    </xsl:template>

    <xsl:template match="a:button">
        <button>
            <xsl:call-template name="processAllAttrs">
                <xsl:with-param name="skipAttrs" select="'label'"/>
            </xsl:call-template>
            <xsl:value-of select="@label"/>
        </button>
    </xsl:template>

    <xsl:template match="a:form">
        <form>
            <xsl:call-template name="processAllAttrs"/>
            <xsl:apply-templates/>
        </form>
    </xsl:template>

    <xsl:template match="a:textarea">
        <textarea>
            <xsl:if test="@cols">
                <xsl:copy-of select="@cols"/>
            </xsl:if>
            <xsl:call-template name="processAllAttrs">
                <xsl:with-param name="skipAttrs" select="'cols'"/>
            </xsl:call-template>
        </textarea>
    </xsl:template>

    <xsl:template match="a:*[util:stringEndsWith(local-name(), 'Model') or local-name() = 'uiController']">
        <!-- Models already processed -->
    </xsl:template>

    <xsl:template match="a:*">
        <div data-widget="{local-name()}">
            <xsl:call-template name="processAllAttrs"/>
            <xsl:apply-templates/>
        </div>
    </xsl:template>

    <xsl:template match="x:* | *[namespace-uri() = '']">
        <xsl:if test="not(util:stringEndsWith(local-name(), 'Model'))">
            <xsl:element name="{local-name()}">
                <xsl:for-each select="@*">
                    <xsl:choose>
                        <xsl:when test="local-name() = 'idref'">
                            <xsl:attribute name="data-idref">
                                <xsl:value-of select="."/>
                            </xsl:attribute>
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:copy/>
                        </xsl:otherwise>
                    </xsl:choose>
                </xsl:for-each>
                <xsl:apply-templates/>
            </xsl:element>
        </xsl:if>
    </xsl:template>

    <xsl:template match="*[namespace-uri() != '']" priority="-1">
        <div data-widget="{local-name()}" data-namespace="{namespace-uri()}">
            <xsl:call-template name="processAllAttrs"/>
            <xsl:apply-templates/>
        </div>
    </xsl:template>

    <xsl:template match="text()">
        <xsl:if test="not(util:stringEndsWith(name(parent::*), 'Model'))">
            <xsl:copy/>
        </xsl:if>
    </xsl:template>

    <xsl:template name="processAllAttrs">
        <xsl:param name="skipAttrs"/>

        <!-- Serialize non-CamelCased attributes first -->
        <xsl:for-each select="@*[not(util:isCamelName(local-name()))]">
            <xsl:if test="not(util:stringContains($skipAttrs, local-name()))">
                <xsl:choose>
                    <xsl:when test="local-name() = 'id'">
                        <xsl:copy/>
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:attribute name="{concat('data-', local-name())}">
                            <xsl:value-of select="."/>
                        </xsl:attribute>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:if>
        </xsl:for-each>

        <!-- Serialize all CamelCased attributes in a single data-props -->
        <xsl:variable name="camelAttrs" select="@*[util:isCamelName(local-name())]"/>
        <xsl:if test="$camelAttrs">
            <xsl:attribute name="data-props">
                <xsl:value-of select="util:serializeProperties($camelAttrs)"/>
            </xsl:attribute>
        </xsl:if>
    </xsl:template>

    <xsl:template match="comment()">
        <xsl:copy/>
    </xsl:template>
    
</xsl:stylesheet>
